﻿using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.backgroundImageFileButton.Click += backgroundImageFileButton_Click;
            this.previewButton.Click             += previewButton_Click;
            this.colorButton.Click               += colorButton_Click;
            this.saveButton.Click                += saveButton_Click;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 배경 이미지 파일 버튼 클릭시 처리하기 - backgroundImageFileButton_Click(sender, e)

        /// <summary>
        /// 배경 이미지 파일 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void backgroundImageFileButton_Click(object sender, EventArgs e)
        {
            if(this.openFileDialog.ShowDialog() == DialogResult.OK)
            {
                this.backgroundImageFileTextBox.Text = this.openFileDialog.FileName;
            }
        }

        #endregion
        #region 미리보기 버튼 클릭시 처리하기 - previewButton_Click(sender, e)

        /// <summary>
        /// 미리보기 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void previewButton_Click(object sender, EventArgs e)
        {
            if(CheckValidation())
            {
                Image image = WatermarkHelper.GetWatermarkImage
                (
                    this.backgroundImageFileTextBox.Text,
                    new Font(Font.FontFamily, 24f),
                    this.opacityHScrollBar.Value,
                    new SolidBrush(this.colorDialog.Color),
                    this.watermarkTextTextBox.Text
                );

                this.pictureBox.Width  = image.Width;
                this.pictureBox.Height = image.Height;
                this.pictureBox.Image  = image;
            }
        }

        #endregion
        #region 색상 버튼 클릭시 처리하기 - colorButton_Click(sender, e)

        /// <summary>
        /// 색상 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void colorButton_Click(object sender, EventArgs e)
        {
            this.colorDialog.ShowDialog();
        }

        #endregion
        #region 저장 버튼 클릭시 처리하기 - saveButton_Click(sender, e)

        /// <summary>
        /// 저장 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void saveButton_Click(object sender, EventArgs e)
        {
            Bitmap bitmap = this.pictureBox.Image as Bitmap;

            if(bitmap == null)
            {
                return;
            }

            if(this.saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                bitmap.Save(saveFileDialog.FileName, ImageFormat.Png);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 무결성 조사하기 - CheckValidation()

        /// <summary>
        /// 무결성 조사하기
        /// </summary>
        /// <returns>무결성 조사 결과</returns>
        private bool CheckValidation()
        {
            if(string.IsNullOrWhiteSpace(this.backgroundImageFileTextBox.Text))
            {
                MessageBox.Show("배경 이미지 파일을 선택하시기 바랍니다.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);

                return false;
            }

            if(string.IsNullOrWhiteSpace(this.watermarkTextTextBox.Text))
            {
                MessageBox.Show("워터마크 텍스트를 입력해 주시기 바랍니다.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);

                return false;
            }

            return true;
        }

        #endregion
    }
}